==> molview_pubchem-0.1.2.tar.gz: lib/src/compounds/find_suggestions.dart <==
// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

part of molview_pubchem.module.compounds;

const pubchemAutocompleteApiRoot =
    'https://pubchem.ncbi.nlm.nih.gov/pcautocp/pcautocp.cgi';
const pubchemCompoundsDict = 'pc_compoundnames';

/// Find compound search suggestions using the PubChem autocomplete API.
///
/// Reference:
/// https://pubchem.ncbi.nlm.nih.gov/widget/docs/widget_autocomplete_help.html
Future<v0.SearchSuggestions> findSuggestions(
    v0.FindSearchSuggestions query, State state) async {
  // Exit right away if the query text is empty.
  if (query.text.isEmpty) {
    return new v0.SearchSuggestions('pubchem', 'compounds', fields.nullKey, []);
  }

  var requestUrl =
      '$pubchemAutocompleteApiRoot?dict=$pubchemCompoundsDict&q=${query.text}&n=${query.count}';

  // Execute request.
  Map<String, dynamic> response =
      JSON.decode((await state.httpClient.get(requestUrl)).body)
      as Map<String, dynamic>;

  // Extract suggestions.
  var suggestions = new List<SearchSuggestion>();
  List<String> autocpArray = response['autocp_array'] as List<String>;
  autocpArray.forEach((String title) => suggestions
      .add(new SearchSuggestion(ucfirst(onlyLowercaseWords(title)), null)));

  return new v0.SearchSuggestions(
      'pubchem', 'compounds', fields.nullKey, suggestions);
}

/// Replace all uppercase-only words with lowercase words.
String onlyLowercaseWords(String src) {
  final regex = new RegExp(r'\b([A-Z]+)\b');
  return src.replaceAllMapped(regex, (Match m) => m.group(1).toLowerCase());
}

/// Make first character uppercase.
String ucfirst(String src) {
  if (src.length > 0) {
    src = src[0].toUpperCase() + src.substring(1);
  }
  return src;
}

==> molview_pubchem-0.2.0.tar.gz: lib/src/compounds/find_suggestions.dart <==
// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

part of molview_pubchem.module.compounds;

const pubchemCompoundsDict = 'pc_compoundnames';

/// Find compound search suggestions using the PubChem autocomplete API.
///
/// Reference:
/// https://pubchem.ncbi.nlm.nih.gov/widget/docs/widget_autocomplete_help.html
Future<v0.SearchSuggestions> findSuggestions(
    v0.FindSearchSuggestions query, State state) async {
  // Exit right away if the PubChem autocp provider is in accept or if the
  // query text is empty.
  if (!query.accept.contains(pubchemAutocpProvider) || query.text.isEmpty) {
    return new v0.SearchSuggestions(pubchemAutocpProvider, []);
  }

  var requestUrl =
      '$pubchemAutocpApiRoot?dict=$pubchemCompoundsDict&q=${query.text}&n=${query.count}';

  // Execute request.
  Map<String, dynamic> response =
      JSON.decode((await state.httpClient.get(requestUrl)).body)
      as Map<String, dynamic>;

  // Extract suggestions.
  var suggestions = new List<SearchSuggestion>();
  List<String> autocpArray = response['autocp_array'] as List<String>;
  autocpArray.forEach((String title) => suggestions.add(new SearchSuggestion(
      ucfirst(onlyLowercaseWords(title)), fields.entityCompound, [])));

  return new v0.SearchSuggestions(pubchemAutocpProvider, suggestions);
}

/// Replace all uppercase-only words with lowercase words.
String onlyLowercaseWords(String src) {
  final regex = new RegExp(r'\b([A-Z]+)\b');
  return src.replaceAllMapped(regex, (Match m) => m.group(1).toLowerCase());
}

/// Make first character uppercase.
String ucfirst(String src) {
  if (src.length > 0) {
    src = src[0].toUpperCase() + src.substring(1);
  }
  return src;
}
